﻿using DevExpress.Utils;
using DevExpress.XtraGrid.Columns;
using DevExpress.XtraGrid.Views.Base;
using DevExpress.XtraGrid.Views.Grid;

namespace TestProject
{
    /// <summary>
    /// 그리드 확장
    /// </summary>
    public static class GridExtension
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 뷰 초기화 하기 - InitializeView(view, multiSelect, gridMultiSelectMode, drawFocusRectStyle, showIndicator, showColumnHeaders, allowColumnMoving, allowColumnResizing, allowFilter, allowSort, allowCellMerge, editorShowMode, editable)

        /// <summary>
        /// 뷰 초기화 하기
        /// </summary>
        /// <param name="view">뷰</param>
        /// <param name="multiSelect">멀티 선택 여부</param>
        /// <param name="gridMultiSelectMode">그리드 멀티 선택 모드</param>
        /// <param name="drawFocusRectStyle">포커스 사각형 그리기 스타일</param>
        /// <param name="showIndicator">지시자 표시 여부</param>
        /// <param name="showColumnHeaders">컬럼 헤더 표시 여부</param>
        /// <param name="allowColumnMoving">컬럼 이동 허용 여부</param>
        /// <param name="allowColumnResizing">컬럼 크기 변경 허용 여부</param>
        /// <param name="allowFilter">필터 허용 여부</param>
        /// <param name="allowSort">정렬 허용 여부</param>
        /// <param name="allowCellMerge">셀 병합 사용 여부</param>
        /// <param name="editorShowMode">에디터 표시 모드</param>
        /// <param name="editable">편집 허용 여부</param>
        public static void InitializeView(this GridView view, bool multiSelect, GridMultiSelectMode gridMultiSelectMode, DrawFocusRectStyle drawFocusRectStyle, bool showIndicator, bool showColumnHeaders, bool allowColumnMoving, bool allowColumnResizing, bool allowFilter, bool allowSort, bool allowCellMerge, EditorShowMode editorShowMode, bool editable)
        {
            view.OptionsDetail.EnableMasterViewMode = false;
            view.OptionsSelection.MultiSelect       = multiSelect;
            view.OptionsSelection.MultiSelectMode   = gridMultiSelectMode;
            view.FocusRectStyle                     = drawFocusRectStyle;

            // 그룹 패널
            view.OptionsView.ShowGroupPanel = false;

            // 행
            view.IndicatorWidth                         = 30;
            view.OptionsView.ShowIndicator              = showIndicator;
            view.OptionsView.EnableAppearanceEvenRow    = false;
            view.OptionsView.EnableAppearanceOddRow     = false;
            view.OptionsBehavior.KeepFocusedRowOnUpdate = false;
            view.OptionsNavigation.AutoFocusNewRow      = true;

            // 컬럼
            view.OptionsBehavior.AutoPopulateColumns      = false;
            view.OptionsView.ShowColumnHeaders            = showColumnHeaders;
            view.OptionsView.ColumnAutoWidth              = false;
            view.OptionsMenu.EnableColumnMenu             = false;
            view.OptionsNavigation.EnterMoveNextColumn    = true;
            view.OptionsCustomization.AllowColumnMoving   = allowColumnMoving;
            view.OptionsCustomization.AllowColumnResizing = allowColumnResizing;
            view.OptionsCustomization.AllowFilter         = allowFilter;
            view.OptionsCustomization.AllowSort           = allowSort;

            // 셀
            view.OptionsView.ShowButtonMode                   = ShowButtonModeEnum.ShowForFocusedCell;
            view.OptionsView.AllowCellMerge                   = allowCellMerge;
            view.OptionsBehavior.EditorShowMode               = editorShowMode;
            view.OptionsBehavior.Editable                     = editable;
            view.OptionsSelection.EnableAppearanceFocusedCell = editable;
            view.OptionsSelection.InvertSelection             = false;
            view.OptionsHint.ShowCellHints                    = false;
            view.OptionsClipboard.CopyColumnHeaders           = DefaultBoolean.True;
        }

        #endregion
        #region 컬럼 추가하기 - AddColumn(view, columnName, fieldName, caption, columnWidth, hAlignment, allowEdit, displayFormatType, displayFormatString)

        /// <summary>
        /// 컬럼 추가하기
        /// </summary>
        /// <param name="view">뷰</param>
        /// <param name="columnName">컬럼명</param>
        /// <param name="fieldName">필드명</param>
        /// <param name="caption">제목</param>
        /// <param name="columnWidth">컬럼 너비</param>
        /// <param name="hAlignment">수평 정렬</param>
        /// <param name="allowEdit">편집 허용 여부</param>
        /// <param name="displayFormatType">디스플레이 포맷 타입</param>
        /// <param name="displayFormatString">디스플레이 포맷 문자열</param>
        /// <returns>그리드 컬럼</returns>
        public static GridColumn AddColumn(this GridView view, string columnName, string fieldName, string caption, int columnWidth, HorzAlignment hAlignment, bool allowEdit, FormatType? displayFormatType, string displayFormatString)
        {
            GridColumn column = view.Columns.AddVisible(fieldName, caption);

            column.Name                                    = columnName;
            column.Width                                   = columnWidth;
            column.AppearanceHeader.Options.UseTextOptions = true;
            column.AppearanceHeader.TextOptions.HAlignment = HorzAlignment.Center;
            column.AppearanceHeader.TextOptions.VAlignment = VertAlignment.Center;
            column.AppearanceCell.Options.UseTextOptions   = true;
            column.AppearanceCell.TextOptions.HAlignment   = hAlignment;
            column.AppearanceCell.TextOptions.VAlignment   = VertAlignment.Center;
            column.OptionsColumn.AllowEdit                 = allowEdit;
            column.ColumnEdit                              = null;

            if(displayFormatType.HasValue)
            {
                column.DisplayFormat.FormatType   = displayFormatType.Value;
                column.DisplayFormat.FormatString = displayFormatString;
            }

            return column;
        }

        #endregion
    }
}